diff -r 6e8bbc150fa9 src/imap-login/client.h
--- a/src/imap-login/client.h	Mon Jun 24 23:33:59 2013 +0300
+++ b/src/imap-login/client.h	Mon Jun 24 23:39:05 2013 +0300
@@ -19,6 +19,7 @@
 	unsigned int cmd_finished:1;
 	unsigned int proxy_sasl_ir:1;
 	unsigned int proxy_seen_banner:1;
+	unsigned int proxy_no_pipelining:1;
 	unsigned int skip_line:1;
 	unsigned int id_logged:1;
 	unsigned int client_ignores_capability_resp_code:1;
diff -r 6e8bbc150fa9 src/imap-login/imap-proxy.c
--- a/src/imap-login/imap-proxy.c	Mon Jun 24 23:33:59 2013 +0300
+++ b/src/imap-login/imap-proxy.c	Mon Jun 24 23:39:05 2013 +0300
@@ -63,8 +63,12 @@
 	unsigned int len;
 	const char *mech_name, *error;
 
-	if (client->proxy_backend_capability == NULL)
+	if (client->proxy_backend_capability == NULL &&
+	    client->common.proxy_state != IMAP_PROXY_STATE_CAPABILITY) {
 		str_append(str, "C CAPABILITY\r\n");
+		if (client->proxy_no_pipelining)
+			return 0;
+	}
 
 	if (client->common.proxy_mech == NULL) {
 		/* logging in normally - use LOGIN command */
@@ -122,6 +126,11 @@
 			str_sanitize(line, 160)));
 		return -1;
 	}
+	if (strstr(line, "Microsoft Exchange") != NULL) {
+		/* Exchange 2013 hangs if we send the CAPABILITY pipelined
+		   with LOGIN */
+		client->proxy_no_pipelining = TRUE;
+	}
 
 	str = t_str_new(128);
 	if (strncmp(line + 5, "[CAPABILITY ", 12) == 0) {
@@ -270,7 +279,7 @@
 			return -1;
 		}
 		o_stream_nsend(output, str_data(str), str_len(str));
-		return 1;
+		return 0;
 	} else if (strncmp(line, "L OK ", 5) == 0) {
 		/* Login successful. Send this line to client. */
 		client->proxy_state = IMAP_PROXY_STATE_LOGIN;
@@ -280,7 +289,7 @@
 
 		(void)client_skip_line(imap_client);
 		client_proxy_finish_destroy_client(client);
-		return 1;
+		return 0;
 	} else if (strncmp(line, "L ", 2) == 0) {
 		line += 2;
 		if (client->set->auth_verbose) {
@@ -328,6 +337,14 @@
 	} else if (strncmp(line, "C ", 2) == 0) {
 		/* Reply to CAPABILITY command we sent, ignore it */
 		client->proxy_state = IMAP_PROXY_STATE_CAPABILITY;
+		if (imap_client->proxy_no_pipelining) {
+			str = t_str_new(128);
+			if (proxy_write_login(imap_client, str) < 0) {
+				client_proxy_failed(client, TRUE);
+				return -1;
+			}
+			o_stream_nsend(output, str_data(str), str_len(str));
+		}
 		return 0;
 	} else if (strncasecmp(line, "I ", 2) == 0 ||
 		   strncasecmp(line, "* ID ", 5) == 0) {
